import { ref, Ref, watch, onUnmounted } from 'vue';
import tickWav from './audio/tick.wav';
import tockWav from './audio/tock.wav';
/*  播放相关 */
export default function useMetronome(
  beatVal: Ref<string>,
  beatSymbol: Ref<string>
) {
  let _timeTask: NodeJS.Timer;
  const playerTick = new Audio(tickWav);
  const playerTock = new Audio(tockWav);
  /* 音量 */
  const volumeNum = ref(100);
  watch(volumeNum, () => {
    playerTick.volume = volumeNum.value / 100;
    playerTock.volume = volumeNum.value / 100;
  });
  /* 播放状态 */
  const playState = ref<'play' | 'pause'>('pause');
  /* 速度 */
  const speedNum = ref(90);

  onUnmounted(() => {
    pausePlay();
  });
  // 开始播放
  function startPlay() {
    playerTick.currentTime = 0;
    playerTock.currentTime = 0;
    const timeArr = computeTimeArr();
    handleBeatPlay(timeArr, speedNum.value);
    playState.value = 'play';
  }
  //暂停播放
  function pausePlay() {
    playerTick.pause();
    playerTock.pause();
    clearTimeout(_timeTask);
    playState.value = 'pause';
  }
  function handleBeatPlay(timeArr: string[], speed: number) {
    let index = 0;
    timeTask();
    function timeTask() {
      const bateNum = Number(timeArr[index]);
      // 当为index=0的时候第一排为重（当为0拍的时候不为重）
      const playVm = index === 0 && bateNum !== 0 ? playerTick : playerTock;
      playVm.play();
      _timeTask = setTimeout(() => {
        playVm.pause();
        playVm.currentTime = 0;
        index === timeArr.length - 1 ? (index = 0) : index++;
        timeTask();
      }, ((1000 * 60) / speed) * (bateNum || 1)); // 为0拍的时候 变为1
    }
  }
  function computeTimeArr() {
    if (beatSymbol.value === '1') {
      return beatVal.value.split('-');
    }
    return beatSymbol.value.split('-');
  }
  return {
    volumeNum,
    playState,
    speedNum,
    startPlay,
    pausePlay
  };
}
